@@ -24,8 +24,7 @@ template< typename Derived, typename Spec >
 struct descender_preferred_order
 : tree_descender< Derived, Spec >
 {
-    typedef Spec                              spec;              ///< Specifications.
-    STEME_TYPEDEF_SEQAN_TYPES( spec )
+    STEME_TYPEDEF_SEQAN_TYPES( Spec )
     typedef tree_descender< Derived, Spec >   descender_t;       ///< Descender base type.
 
     typedef std::vector< alphabet_t >         dna_vec;           ///< Vector of DNA type.
@@ -41,7 +40,7 @@ struct descender_preferred_order
     : descender_t( _data, _model )
     {
         preferred_bases.resize( _model.W() );
-        for( size_t w = 0; _model.W() != w; ++w ) {
+        for( size_t w = 0; size_t( _model.W() ) != w; ++w ) {
             calculate_preferred_bases_column( _model.bs.pssm.log_probs.m[ w ], preferred_bases[ w ] );
         }
     }
@@ -59,7 +58,7 @@ struct descender_preferred_order
 
         // build a vector that we can sort so most likely base is first
         to_sort_t to_sort;
-        for( int i = 0; boost::size( col ) != i; ++i ) {
+        for( int i = 0; int( boost::size( col ) ) != i; ++i ) {
             to_sort.push_back( boost::make_tuple( col[ i ], alphabet_t( i ) ) );
         }
         std::sort( to_sort.begin(), to_sort.end(), std::greater< sortable_t >() );
