@@ -51,7 +51,7 @@ struct EM_descender
 	double                     log_delta_factor;         ///< The amount by which we change log delta when we see the estimation of c_wb is too low.
 	double                     log_delta_adjustment_max; ///< The largest adjustment to log delta we allow.
 	double                     log_delta_adjustment;     ///< An adjustment we make to log delta for the next iteration after seeing the actual value of c_wb. This can change every iteration.
-	double                     log_delta;                ///< Threshold on log expected Z_n.
+	double                     llr_threshold;            ///< Threshold on log likelihood ratio.
 	double                     wnsites;                  ///< Weight on number of sites. Used when updating lambda.
 	bool                       using_sparse_Z;           ///< Are we using a sparse Z?
 	Z_sparse_vec               Z_sparse;                 ///< The evaluations (expected Zs) of each W-mer under the binding site model.
@@ -83,7 +83,7 @@ struct EM_descender
 	, log_delta_factor( log_delta_factor )
 	, log_delta_adjustment_max( log_delta_adjustment_max )
 	, log_delta_adjustment( 0. )
-	, log_delta( -std::numeric_limits< double >::max() )
+	, llr_threshold( -std::numeric_limits< double >::max() )
 	, wnsites( wnsites )
 	, using_sparse_Z( false )
 	, Z_sparse( _data.N, _data.N / 3 )
@@ -119,7 +119,8 @@ struct EM_descender
 	void
 	on_descent_begin() {
 		// set the threshold
-		log_delta = std::log( epsilon ) + base_t::_model._prior.lambda.get_log_lambda();
+	    const double delta_threshold = epsilon * base_t::_model._prior.lambda.get_lambda();
+		llr_threshold = std::log( delta_threshold ) - std::log( 1. - delta_threshold );
 		estimated_c_wb = base_t::_model._prior.lambda.get_lambda() * base_t::_data.get_occurrence_count( base_t::_model.W() );
 
 		//reset Z estimates
@@ -206,11 +207,11 @@ struct EM_descender
 
 
 	/**
-	 * \return The threshold on log(expected[Z]).
+	 * \return The threshold on log likelihood ratio.
 	 */
 	double
-	get_log_Z_threshold() {
-		return log_delta;
+	get_llr_threshold() {
+		return llr_threshold;
 	}
 
 
