@@ -845,7 +845,7 @@ struct find_best_w_mers
 	container_ptr                        best_w_mers;      ///< Best w_mers.
 	storage_policy                       storage;          ///< Stores the best W-mers.
 	double                              worst_Z;          ///< The worst Z so far.
-	double                              lZt;             ///< The log Z threshold term.
+	double                              llr_threshold;    ///< The log likelihood ratio threshold.
 
 
 	/** Constructor. */
@@ -868,18 +868,18 @@ struct find_best_w_mers
 	void
 	on_descent_begin() {
 		base_t::stats.reset();
-		worst_Z = lZt = -std::numeric_limits< double >::max();
+		worst_Z = llr_threshold = -std::numeric_limits< double >::max();
 		storage.initialise();
 	}
 
 
 	/**
-	 * \return The threshold on log(<Z>).
+	 * \return The threshold on log likelihood ratio.
 	 */
 	double
-	get_log_Z_threshold() {
+	get_llr_threshold() {
 		if( storage.stored_enough() ) {
-			return lZt;
+			return llr_threshold;
 		} else {
 			return -std::numeric_limits< double >::max();
 		}
@@ -894,7 +894,7 @@ struct find_best_w_mers
 	update_worst_Z( double new_worst_Z ) {
 		if( new_worst_Z != worst_Z ) {
 			worst_Z = new_worst_Z;
-			lZt = std::log( new_worst_Z );
+			llr_threshold = std::log( new_worst_Z / ( 1. - new_worst_Z ) );
 		}
 	}
 
