@@ -7,6 +7,9 @@
 
 #include <steme/background_model.h>
 
+#include <boost/static_assert.hpp>
+#include <boost/type_traits.hpp>
+
 namespace py = boost::python;
 using namespace steme;
 using namespace steme::python;
@@ -17,9 +20,10 @@ typedef seqan::Value< fibre_sa_t, 1 >::Type seq_index_t;
 typedef seqan::Value< fibre_sa_t, 2 >::Type seq_offset_t;
 typedef seqan::Value< seqan::Fibre< seqan_types_t::index_t, seqan::FibreChildtab >::Type >::Type text_offset_t;
 
-static const unsigned long long index_max_seqs = ( unsigned long long )( std::numeric_limits< seq_index_t >::max() ) + 1;
-static const unsigned long long index_max_seq_length = ( unsigned long long )( std::numeric_limits< seq_offset_t >::max() ) + 1;
-static const unsigned long long index_max_text_length = ( unsigned long long )( std::numeric_limits< text_offset_t >::max() ) + 1;
+typedef unsigned long long max_representing_t;
+static const max_representing_t index_max_seqs = ( max_representing_t )( std::numeric_limits< seqan_types_t::seq_index_t >::max() ) + 1;
+static const max_representing_t index_max_seq_length = ( max_representing_t )( std::numeric_limits< seqan_types_t::seq_offset_t >::max() ) + 1;
+static const max_representing_t index_max_text_length = ( max_representing_t )( std::numeric_limits< seqan_types_t::global_pos_t >::max() ) + 1;
 
 
 /**
@@ -114,7 +118,7 @@ build_index( const seqan_types_t::string_set_t & sequences )
     if( index_max_seqs < length( sequences ) ) {
         throw std::length_error( "Too many sequences for this index type." );
     }
-    unsigned long long total = 0;
+    max_representing_t total = 0;
     BOOST_FOREACH( const seqan_types_t::string_t & seq, sequences ) {
         if( index_max_seq_length < length( seq ) ) {
             throw std::length_error( "Sequence too long for this index type." );
@@ -185,6 +189,10 @@ register_seqan_string_to_python() {
 }
 
 
+template< typename T >
+void dummy() {
+    T t = "";
+}
 
 
 
