@@ -16,6 +16,15 @@
 #include <iostream>
 #include <limits>
 
+#include <seqan/version.h>
+#if SEQAN_VERSION_MAJOR > 1 || SEQAN_VERSION_MINOR > 3
+# define SEQAN_PACK Pack
+# define SEQAN_BITPACKED BitPacked
+#else
+# define SEQAN_PACK Compressed
+# define SEQAN_BITPACKED BitCompressed
+#endif
+
 using namespace seqan;
 using namespace std;
 
@@ -57,12 +66,12 @@ namespace seqan
          * of maximum length numeric_limits< offset_t >::max() base pairs
          */
         typedef String<
-                        Pair< seq_index_t, offset_t, Compressed >,
+                        Pair< seq_index_t, offset_t, SEQAN_PACK >,
                         DefaultIndexStringSpec< index_t >::Type
                        > Type;
 
         // Use a mmapped string
-        // typedef String< Pair<unsigned char, unsigned int, Compressed>, MMap<> >   Type;
+        // typedef String< Pair<unsigned char, unsigned int, SEQAN_PACK>, MMap<> >   Type;
     };
 
 
