@@ -8,11 +8,14 @@ Code to produce MEME-like output from STEME.
 """
 
 
-import numpy as N, os, sys
+import numpy as N
+import os
+import sys
 from itertools import imap
 
 
 class MemeLikeOutput(object):
+
     """
     Produces MEME-like output that can be parsed by biopython, bioperl or MAST parsers.
     """
@@ -20,7 +23,7 @@ class MemeLikeOutput(object):
     def __init__(self, f):
         self.f = f
         "The output file handle."
-        
+
     def initialise(self, algorithm):
         """
         Write the version
@@ -42,8 +45,6 @@ for interactive use and downloading at http://meme.nbcr.net.
 
 """ % algorithm.version())
 
-
-
         #
         # Write the reference info
         #
@@ -59,8 +60,6 @@ Reid JE, Wernisch L.
 ********************************************************************************
 """)
 
-
-
         #
         # Write the sequence info if we don't have too many
         #
@@ -78,16 +77,14 @@ Sequence name            Weight Length  Sequence name            Weight Length
 
 """ % (
                 algorithm.fasta,
-                '\n'.join("%-24s 1.0000 %6d" % (_id, algorithm.input_sequences.seqs.string_length(i)) for i, _id in enumerate(algorithm.input_sequences.ids))
+                '\n'.join("%-24s 1.0000 %6d" % (_id, algorithm.input_sequences.seqs.string_length(i))
+                          for i, _id in enumerate(algorithm.input_sequences.ids))
             ))
 
-
-
-
         #
         # Write the command line summary
         #
-        # need to add revcomp to command summary as BioPython looks for it to 
+        # need to add revcomp to command summary as BioPython looks for it to
         # determine if using both strands
         self.f.write("""
 ********************************************************************************
@@ -116,45 +113,45 @@ A %.3f C %.3f G %.3f T %.3f
 ********************************************************************************
 
 """ % (
-    ' '.join(sys.argv),
-    algorithm.options.num_motifs,
-    algorithm.options.min_w,
-    algorithm.options.max_w,
-    algorithm.options.min_num_sites,
-    algorithm.options.max_num_sites,
-    algorithm.options.wnsites,
-    algorithm.options.starts_seed_pseudo_counts,
-    algorithm.options.max_iters,
-    algorithm.options.convergence_distance,
-    algorithm.input_sequences.data.N,
-    len(algorithm.input_sequences.seqs),
-    algorithm.freqs.freq(0),
-    algorithm.freqs.freq(1),
-    algorithm.freqs.freq(2),
-    algorithm.freqs.freq(3),
-    algorithm.freqs_with_pseudo_counts.freq(0),
-    algorithm.freqs_with_pseudo_counts.freq(1),
-    algorithm.freqs_with_pseudo_counts.freq(2),
-    algorithm.freqs_with_pseudo_counts.freq(3),
-))
-        
-
-
+                     ' '.join(sys.argv),
+                     algorithm.options.num_motifs,
+                     algorithm.options.min_w,
+                     algorithm.options.max_w,
+                     algorithm.options.min_num_sites,
+                     algorithm.options.max_num_sites,
+                     algorithm.options.wnsites,
+                     algorithm.options.starts_seed_pseudo_counts,
+                     algorithm.options.max_iters,
+                     algorithm.options.convergence_distance,
+                     algorithm.input_sequences.data.N,
+                     len(algorithm.input_sequences.seqs),
+                     algorithm.freqs.freq(0),
+                     algorithm.freqs.freq(1),
+                     algorithm.freqs.freq(2),
+                     algorithm.freqs.freq(3),
+                     algorithm.freqs_with_pseudo_counts.freq(0),
+                     algorithm.freqs_with_pseudo_counts.freq(1),
+                     algorithm.freqs_with_pseudo_counts.freq(2),
+                     algorithm.freqs_with_pseudo_counts.freq(3),
+                     ))
 
     def write_motif(self, algorithm, motif):
         """
         Write the motif
         """
-        rounded_pspm = (N.exp(motif.model.bs.pssm.log_probs.values()) * 10.).round().astype(int)
-        
+        rounded_pspm = (
+            N.exp(motif.model.bs.pssm.log_probs.values()) * 10.).round().astype(int)
+
         def char(x):
-            if 0 == x: return ':'
-            if 10 == x: return 'a'
+            if 0 == x:
+                return ':'
+            if 10 == x:
+                return 'a'
             return str(x)
-        
+
         def pssm_line(b):
-            return ''.join(imap(char, rounded_pspm[:,b]))
-        
+            return ''.join(imap(char, rounded_pspm[:, b]))
+
         self.f.write("""
 ********************************************************************************
 MOTIF %2d    width = %4d   sites = %3d   llr = %f   E-value = %e
@@ -168,12 +165,9 @@ probability       G  %s
 matrix            T  %s
 
 --------------------------------------------------------------------------------
-""" % (motif.idx+1, motif.model.W, motif.num_sites, motif.LLR, N.exp(motif.log_E_value), motif.idx+1,
-    pssm_line(0), pssm_line(1), pssm_line(2), pssm_line(3)))
-        
-        
-        
-        
+""" % (motif.idx + 1, motif.model.W, motif.num_sites, motif.LLR, N.exp(motif.log_E_value), motif.idx + 1,
+                    pssm_line(0), pssm_line(1), pssm_line(2), pssm_line(3)))
+
     def write_relative_entropy(self, algorithm, motif):
         """
         Write the relative entropy - NOT IMPLEMENTED
@@ -193,10 +187,6 @@ Entropy          1.0 **** ***
                  0.0 --------
 """)
 
-        
-        
-        
-        
     def write_multilevel_consensus(self, algorithm, motif):
         """
         Write the multilevel consensus sequence - NOT IMPLEMENTED
@@ -208,9 +198,6 @@ consensus             T  A A
 sequence                 T                             
 """)
 
-
-
-
     def write_sites(self, algorithm, motif):
         """
         Write something like...
@@ -227,13 +214,11 @@ Sequence name            Strand  Start   P-value              Site
 %s
 --------------------------------------------------------------------------------
 """ % (
-                motif.idx+1,
-                '\n'.join([self._output_for_prediction(algorithm, prediction) for prediction in motif.predictions])
+                motif.idx + 1,
+                '\n'.join([self._output_for_prediction(algorithm, prediction)
+                          for prediction in motif.predictions])
             ))
 
-
-
-
     def _output_for_prediction(self, algorithm, prediction, site_flanks=10):
         """
         Sequence name            Strand  Start   P-value              Site
@@ -243,17 +228,26 @@ Sequence name            Strand  Start   P-value              Site
         from . import reverse_complement
         seq_index = algorithm.input_sequences.index_for_seq_id(prediction.seq)
         flank_start = max(prediction.interval.start - site_flanks, 0)
-        flank_end = min(prediction.interval.end + site_flanks, algorithm.input_sequences.data.seq_length(seq_index))
-        left_flank = algorithm.input_sequences.data.subsequence(seq_index, flank_start, prediction.interval.start - flank_start)
-        site = algorithm.input_sequences.data.subsequence(seq_index, prediction.interval.start, prediction.interval.end - prediction.interval.start)
-        right_flank = algorithm.input_sequences.data.subsequence(seq_index, prediction.interval.end, flank_end - prediction.interval.end)
+        flank_end = min(prediction.interval.end + site_flanks,
+                        algorithm.input_sequences.data.seq_length(seq_index))
+        left_flank = algorithm.input_sequences.data.subsequence(
+            seq_index, flank_start, prediction.interval.start - flank_start)
+        site = algorithm.input_sequences.data.subsequence(
+            seq_index, prediction.interval.start, prediction.interval.end - prediction.interval.start)
+        right_flank = algorithm.input_sequences.data.subsequence(
+            seq_index, prediction.interval.end, flank_end - prediction.interval.end)
         if prediction.rev_comp:
-            left_flank, right_flank = reverse_complement(right_flank), reverse_complement(left_flank)
+            left_flank, right_flank = reverse_complement(
+                right_flank), reverse_complement(left_flank)
             site = reverse_complement(site)
-        if not left_flank: left_flank = '.'
-        if not right_flank: right_flank = '.'
+        if not left_flank:
+            left_flank = '.'
+        if not right_flank:
+            right_flank = '.'
         return "%-24s %5s %6d %9.1e %*s %s %-*s" % (
-            prediction.seq.split()[0], # only print first part of sequence name, otherwise parsing can get confused later
+            # only print first part of sequence name, otherwise parsing can get
+            # confused later
+            prediction.seq.split()[0],
             prediction.rev_comp and '-' or '+',
             prediction.interval.start,
             prediction.p_value,
@@ -261,10 +255,7 @@ Sequence name            Strand  Start   P-value              Site
             site,
             site_flanks, right_flank,
         )
-    
-    
-    
-    
+
     def write_block_diagrams(self, algorithm, motif):
         """
         Write block diagrams - NOT IMPLEMENTED
@@ -290,10 +281,7 @@ FR0000534                          0.0001  305_[+1]_179
 FR0000781                         0.00012  248_[+1]_19
 FR0000861                          0.0002  161_[-1]_420
 --------------------------------------------------------------------------------
-""" % (motif.idx+1,))
-
-
-
+""" % (motif.idx + 1,))
 
     def write_blocks(self, algorithm, motif):
         """
@@ -331,17 +319,14 @@ FR0000804                (  347) CTGCTCAG  1
 //
 
 --------------------------------------------------------------------------------
-""" % (motif.idx+1, motif.idx+1))
-
-
-
-
+""" % (motif.idx + 1, motif.idx + 1))
 
     def write_pssm(self, algorithm, motif):
         """
         Write position-specific-scoring-matrix
         """
-        int_log_odds = (motif.model.bs.pssm.log_probs.values() * 100).round().astype(int)
+        int_log_odds = (
+            motif.model.bs.pssm.log_probs.values() * 100).round().astype(int)
         self.f.write("""
 --------------------------------------------------------------------------------
     Motif %d position-specific scoring matrix
@@ -350,15 +335,14 @@ log-odds matrix: alength= 4 w= %2d n= %5d bayes= %8.5f E= %.1e
 %s
 --------------------------------------------------------------------------------
 """ % (
-                motif.idx+1, motif.model.W, motif.model.num_W_mers,
-                N.log2((1-motif.model.lambda_)/motif.model.lambda_), N.exp(motif.log_E_value),
-                '\n'.join(''.join('%6d' % l for l in int_log_odds_col) for int_log_odds_col in int_log_odds)
+                motif.idx + 1, motif.model.W, motif.model.num_W_mers,
+                N.log2((1 - motif.model.lambda_) / motif.model.lambda_), N.exp(
+                    motif.log_E_value),
+                '\n'.join(''.join('%6d' % l for l in int_log_odds_col)
+                          for int_log_odds_col in int_log_odds)
             )
         )
 
-
-
-
     def write_pspm(self, algorithm, motif):
         """
         Write position-specific-probability-matrix
@@ -372,15 +356,10 @@ letter-probability matrix: alength= 4 w= %2d nsites= %3d E= %.1e
 %s
 --------------------------------------------------------------------------------
 """ % (
-    motif.idx+1, motif.model.W, motif.num_sites, N.exp(motif.log_E_value),
+    motif.idx + 1, motif.model.W, motif.num_sites, N.exp(motif.log_E_value),
     '\n'.join(''.join('%10f' % l for l in pspm_col) for pspm_col in pspm)
 ))
 
-
-
-
-
-
     def write_regex(self, algorithm, motif):
         """
         Not implemented
@@ -392,32 +371,29 @@ letter-probability matrix: alength= 4 w= %2d nsites= %3d E= %.1e
 --------------------------------------------------------------------------------
 C[AT]GC[CAT]C[TA]G
 --------------------------------------------------------------------------------
-""" % (motif.idx+1,))
-
-
+""" % (motif.idx + 1,))
 
     def found_motif(self, algorithm, motif, seconds_taken):
         self.write_motif(algorithm, motif)
-        #self.write_relative_entropy(algorithm, motif) # not implemented yet
-        #self.write_multilevel_consensus(algorithm, motif) # not implemented yet
+        # self.write_relative_entropy(algorithm, motif) # not implemented yet
+        # self.write_multilevel_consensus(algorithm, motif) # not implemented
+        # yet
         self.write_sites(algorithm, motif)
-        #self.write_block_diagrams(algorithm, motif) # not implemented yet
-        #self.write_blocks(algorithm, motif) # not implemented yet
+        # self.write_block_diagrams(algorithm, motif) # not implemented yet
+        # self.write_blocks(algorithm, motif) # not implemented yet
         self.write_pssm(algorithm, motif)
         self.write_pspm(algorithm, motif)
-        #self.write_regex(algorithm, motif) # not implemented yet
+        # self.write_regex(algorithm, motif) # not implemented yet
         self.f.write("""
 Time  %f secs.
 
 ********************************************************************************
 """ % seconds_taken)
-        # ensure that all internal buffers associated with self.f are written to disk
+        # ensure that all internal buffers associated with self.f are written
+        # to disk
         self.f.flush()
         os.fsync(self.f.fileno())
-    
-    
-    
-    
+
     def write_combined_block_diagrams(self, algorithm, motifs):
         """
         Write combined block diagrams - NOT IMPLEMENTED
@@ -436,8 +412,6 @@ FR0000679                        1.75e-01  360
 --------------------------------------------------------------------------------
 """)
 
-
-
     def finalise(self, algorithm):
         """
         Write an empty summary block. Need to implement this as pycogent does
@@ -469,8 +443,3 @@ CPU: %s
 ********************************************************************************
 """ % (algorithm.options.num_motifs, nodename))
         self.f.close()
-
-
-
-
-
