@@ -6,13 +6,15 @@
 Code to choose whether to import debug or release build of stempy module.
 """
 
-import sys, logging
+import sys
+import logging
 logger = logging.getLogger(__name__)
 
 #
 # decide whether to import debug or release stempy C++ extension
 #
-_python_debug_build = hasattr(sys, "gettotalrefcount") # only available in python debug build
+# only available in python debug build
+_python_debug_build = hasattr(sys, "gettotalrefcount")
 if _python_debug_build:
 
     from ._debug_build import *
@@ -30,4 +32,3 @@ else:
         from _release_build import __google_profiler_start, __google_profiler_stop
 
 logger.info('Loaded STEME C++-python interface from %s', S.__name__)
-
