@@ -1,5 +1,5 @@
 #
-# Copyright John Reid 2012
+# Copyright John Reid 2012, 2013
 #
 
 
@@ -70,7 +70,8 @@ Example of format:
 """
 
 
-from pyparsing import Suppress, CaselessLiteral, ZeroOrMore, OneOrMore, Group, Literal, Keyword, \
+from pyparsing import Suppress, CaselessLiteral, ZeroOrMore, OneOrMore, Group, \
+    Literal, Keyword, \
     Word, Optional, Combine, Empty, White, \
     alphas, printables, nums, restOfLine
 from numpy import asarray
@@ -81,18 +82,22 @@ def K(literal):
     return Suppress(Keyword(literal))
 
 comment = ZeroOrMore(Suppress(Literal('#') + restOfLine))
-point = Literal( "." )
-e = CaselessLiteral( "E" )
+point = Literal(".")
+e = CaselessLiteral("E")
 fnumber = Combine(
-    Word( "+-"+nums, nums ) + 
-    Optional( point + Optional( Word( nums ) ) ) +
-    Optional( e + Word( "+-"+nums, nums ) )
+    Word("+-" + nums, nums) +
+    Optional(point + Optional(Word(nums))) +
+    Optional(e + Word("+-" + nums, nums))
 ) | Literal('inf')
 version = K('MEME version') + Word(printables)('VERSION')
 alphabet = Group(K('ALPHABET=') + Word(alphas))('ALPHABET')
-strands = Group(K('strands:') + Optional(Keyword('+')) + Optional(Keyword('-')))('STRANDS')
-background_freqs = Group(K('Background letter frequencies')  + Suppress(restOfLine)
-    + K('A') + fnumber + K('C') + fnumber + K('G') + fnumber + K('T') + fnumber)('BACK_FREQS')
+strands = Group(K('strands:')
+                + Optional(Keyword('+')) + Optional(Keyword('-')))('STRANDS')
+background_freqs = Group(K('Background letter frequencies')
+                         + Suppress(restOfLine)
+                         + K('A') + fnumber + K('C') + fnumber + K('G')
+                         + fnumber + K('T')
+                         + fnumber)('BACK_FREQS')
 matrix_row = Group(
     fnumber + fnumber + fnumber + fnumber
 )
@@ -126,44 +131,56 @@ meme_format = \
     + comment \
     + Group(OneOrMore(motif))('MOTIFS') \
 
+
 Parsed = namedtuple('Parsed', 'version alphabet strands back_freqs motifs')
 Motif = namedtuple('Motif', 'name altname letter_probs log_odds url')
 Matrix = namedtuple('Matrix', 'alength w nsites E values')
 
+
 def extract_or_none(thing, name, extractor=lambda x: x):
-    """Try to extract named value out of thing but return None if it is not there.
+    """Try to extract named value out of thing but return None if it is not
+    there.
     """
     if name in thing:
         return extractor(thing[name])
     return None
 
+
 def extract_motif(motif):
     return Motif(
         motif['NAME'],
         altname=extract_or_none(motif, 'ALTNAME', str.strip),
-        letter_probs=extract_or_none(motif['MATRICES'], 'LETTER_PROBS', extract_matrix),
-        log_odds=extract_or_none(motif['MATRICES'], 'LOG_ODDS', extract_matrix),
+        letter_probs=extract_or_none(
+            motif['MATRICES'],
+            'LETTER_PROBS',
+            extract_matrix),
+        log_odds=extract_or_none(
+            motif['MATRICES'], 'LOG_ODDS', extract_matrix),
         url=extract_or_none(motif, 'URL')
     )
 
+
 def extract_matrix(matrix):
     return Matrix(
-        alength=extract_or_none(matrix, 'ALENGTH', int), 
-        w=extract_or_none(matrix, 'W', int), 
-        nsites=extract_or_none(matrix, 'NSITES', int), 
-        E=extract_or_none(matrix, 'E', float), 
+        alength=extract_or_none(matrix, 'ALENGTH', int),
+        w=extract_or_none(matrix, 'W', int),
+        nsites=extract_or_none(matrix, 'NSITES', int),
+        E=extract_or_none(matrix, 'E', float),
         values=asarray([map(float, row) for row in matrix['ROWS']]),
     )
-    
+
+
 def extract_parsed(parsed):
     return Parsed(
         parsed['VERSION'],
         alphabet=extract_or_none(parsed, 'ALPHABET', lambda x: x[0]),
         strands=extract_or_none(parsed, 'STRANDS', lambda x: set(x)),
-        back_freqs=extract_or_none(parsed, 'BACK_FREQS', lambda x: map(float, x)),
-        motifs=extract_or_none(parsed, 'MOTIFS', lambda x: map(extract_motif, x))
+        back_freqs=extract_or_none(
+            parsed, 'BACK_FREQS', lambda x: map(float, x)),
+        motifs=extract_or_none(
+            parsed, 'MOTIFS', lambda x: map(extract_motif, x))
     )
-        
+
 
 def do_parse_and_extract(string):
     """Parse and extract info from string."""
@@ -197,28 +214,28 @@ def write_pwm(out, motif):
          0.352941  0.058824  0.000000  0.588235 
     """
     print >>out, \
-"""
+        """
 MOTIF %s %s
 letter-probability matrix: alength= %d w= %d nsites= %d %s
 %s
 """ % (
-    motif.name, motif.altname,
-    motif.letter_probs.alength,
-    motif.letter_probs.w,
-    motif.letter_probs.nsites,
-    motif.letter_probs.E and 'E= %e' % motif.letter_probs.E or '',
-    '\n'.join(
-        ' '.join(('%.6f' % p) for p in row)
-        for row in motif.letter_probs.values
+        motif.name, motif.altname,
+        motif.letter_probs.alength,
+        motif.letter_probs.w,
+        motif.letter_probs.nsites,
+        motif.letter_probs.E and 'E= %e' % motif.letter_probs.E or '',
+        '\n'.join(
+            ' '.join(('%.6f' % p) for p in row)
+            for row in motif.letter_probs.values
+        )
     )
-)
 
 
 def write_pwms(out, meme_info):
     """Write the motifs out in the same format.
     """
     print >>out, \
-"""
+        """
 MEME version %s
 
 ALPHABET= %s
@@ -229,14 +246,14 @@ Background letter frequencies
 A %.3f C %.3f G %.3f T %.3f
 
 """ % (
-    meme_info.version,
-    meme_info.alphabet,
-    ' '.join(meme_info.strands),
-    meme_info.back_freqs[0], meme_info.back_freqs[1], meme_info.back_freqs[2], meme_info.back_freqs[3], 
-)
+        meme_info.version,
+        meme_info.alphabet,
+        ' '.join(meme_info.strands),
+        meme_info.back_freqs[0], meme_info.back_freqs[
+            1], meme_info.back_freqs[2], meme_info.back_freqs[3],
+    )
     for motif in meme_info.motifs:
         write_pwm(out, motif)
-    
 
 
 if '__main__' == __name__:
@@ -244,7 +261,6 @@ if '__main__' == __name__:
     # Parse the sample file
     #
     from pkg_resources import resource_string
-    extracted = do_parse_and_extract(resource_string('stempy', 'test/sample-dna-motif.meme-io'))
+    extracted = do_parse_and_extract(
+        resource_string('stempy', 'test/sample-dna-motif.meme-io'))
     print extracted
-            
-
