@@ -12,12 +12,16 @@ logger = logging.getLogger(__name__)
 
 from cookbook.named_tuple import namedtuple
 from scipy.special import gammaln, digamma
-import pyicl, numpy, pylab, os
+import pyicl
+import numpy
+import pylab
+import os
 from collections import defaultdict
 from .scan import footprint
 
 PairOccurrence = namedtuple('PairOccurrence', 'spacing seq pos strand')
-Spacing = namedtuple('Spacing', 'primary secondary same_strand upstream distance')
+Spacing = namedtuple(
+    'Spacing', 'primary secondary same_strand upstream distance')
 
 
 def add_max_distance_option(parser):
@@ -37,7 +41,7 @@ def add_options(parser):
     from . import scan
     scan.add_options(parser)
     add_max_distance_option(parser)
-    
+
 
 def spacing_idx(max_distance, distance, upstream, same_strand):
     """Calculates an index into a spacing array.
@@ -51,7 +55,7 @@ def spacing_idx(max_distance, distance, upstream, same_strand):
     Returns:
         An index in [0, 4*(max_distance+1)].
     """
-    offset = max_distance+1
+    offset = max_distance + 1
     if distance not in pyicl.IntInterval(0, offset):
         raise ValueError('Bad distance')
     assert offset in pyicl.IntInterval(0, 2 * offset)
@@ -60,11 +64,10 @@ def spacing_idx(max_distance, distance, upstream, same_strand):
     return idx
 
 
-
 def spacing_from_idx(max_distance, idx):
     """Return the spacing for this index.
     """
-    offset = max_distance+1
+    offset = max_distance + 1
     if idx not in pyicl.IntInterval(0, 4 * offset):
         raise ValueError('Bad index')
     same_strand = idx >= (2 * offset)
@@ -78,23 +81,24 @@ def spacing_from_idx(max_distance, idx):
 def _test_idx_fns(max_distance):
     for distance, upstream, same_strand in (
         (0, False, False),
-        (0, True , False),
-        (0, False, True ),
-        (0, True , True ),
+        (0, True, False),
+        (0, False, True),
+        (0, True, True),
         (1, False, False),
-        (1, True , False),
-        (1, False, True ),
-        (1, True , True ),
-        (max_distance-1, False, False),
-        (max_distance-1, True , False),
-        (max_distance-1, False, True ),
-        (max_distance-1, True , True ),
-        (max_distance  , False, False),
-        (max_distance  , True , False),
-        (max_distance  , False, True ),
-        (max_distance  , True , True ),
+        (1, True, False),
+        (1, False, True),
+        (1, True, True),
+        (max_distance - 1, False, False),
+        (max_distance - 1, True, False),
+        (max_distance - 1, False, True),
+        (max_distance - 1, True, True),
+        (max_distance, False, False),
+        (max_distance, True, False),
+        (max_distance, False, True),
+        (max_distance, True, True),
     ):
-        assert (distance, upstream, same_strand) == spacing_from_idx(max_distance, spacing_idx(max_distance, distance, upstream, same_strand))
+        assert (distance, upstream, same_strand) == spacing_from_idx(
+            max_distance, spacing_idx(max_distance, distance, upstream, same_strand))
 
 
 # Uncomment to test index functions
@@ -112,14 +116,13 @@ def can_be_primary(max_distance, primary_footprint, secondary_len, seq_length):
 
 
 def ln_factorial(n):
-    return gammaln(n+1)
+    return gammaln(n + 1)
 
 
 def calc_ln_n_choose(obs):
     """Calculates the log of n choose the observed counts.
     """
-    return ln_factorial(obs.sum()) - ln_factorial(obs).sum();
-
+    return ln_factorial(obs.sum()) - ln_factorial(obs).sum()
 
 
 def calc_multinomial_ln_likelihood_uniform_dist(obs):
@@ -128,7 +131,7 @@ def calc_multinomial_ln_likelihood_uniform_dist(obs):
     n = obs.sum()
     if 0 == n:
         return .0
-    return calc_ln_n_choose(obs) - n * numpy.log(len(obs));
+    return calc_ln_n_choose(obs) - n * numpy.log(len(obs))
 
 
 def calc_ln_gamma_factor(obs, alpha):
@@ -148,11 +151,12 @@ def calc_multinomial_ln_likelihood_dirichlet_prior(obs, alpha):
     return calc_ln_gamma_factor(obs, alpha) + calc_ln_n_choose(obs)
 
 
-#def calc_llr_statistic(obs, alpha):
+# def calc_llr_statistic(obs, alpha):
 #    """Calculates the ratio of evidence in favour of a Dirichlet prior
-#    defined by the alphas over a uniform distribution. 
+#    defined by the alphas over a uniform distribution.
 #    """
-#    return (obs * (digamma(alpha) + numpy.log(4 * len(alpha)))).sum() - obs.sum() * digamma(alpha.sum()) 
+# return (obs * (digamma(alpha) + numpy.log(4 * len(alpha)))).sum() -
+# obs.sum() * digamma(alpha.sum())
 
 
 def calc_llr_statistic(obs, alpha):
@@ -162,9 +166,7 @@ def calc_llr_statistic(obs, alpha):
     N = obs.sum()
     alpha0 = alpha.sum()
     D = len(obs)
-    return gammaln(alpha0) - gammaln(alpha0+N) + (gammaln(alpha+obs) + gammaln(alpha)).sum() + N*numpy.log(D)
-
-
+    return gammaln(alpha0) - gammaln(alpha0 + N) + (gammaln(alpha + obs) + gammaln(alpha)).sum() + N * numpy.log(D)
 
 
 def plot_spacings(max_distance, spacing_counts):
@@ -179,16 +181,18 @@ def plot_spacings(max_distance, spacing_counts):
             if not same_strand:
                 pylab.xlabel(upstream and 'upstream' or 'downstream')
             if not upstream:
-                x = numpy.arange(0, max_distance+1)
+                x = numpy.arange(0, max_distance + 1)
                 ax.yaxis.set_label_position('right')
                 ax.yaxis.tick_right()
             else:
-                pylab.ylabel(same_strand and 'Same strand' or 'Opposite orientation')
+                pylab.ylabel(
+                    same_strand and 'Same strand' or 'Opposite orientation')
                 x = numpy.arange(-max_distance, 1)
-            y = [spacing_counts[spacing_idx(max_distance, int(abs(distance)), upstream, same_strand)] for distance in x]
+            y = [spacing_counts[spacing_idx(max_distance, int(abs(distance)), upstream, same_strand)]
+                 for distance in x]
             pylab.bar(x, y, align='center', width=.5, color='grey')
             pylab.ylim(0, ymax)
-            pylab.xlim(x[0]-.5, x[-1]+.5)
+            pylab.xlim(x[0] - .5, x[-1] + .5)
             ax.yaxis.set_major_locator(pylab.MaxNLocator(integer=True))
             ax.xaxis.set_major_locator(pylab.MaxNLocator(integer=True))
             subplot = subplot + 1
@@ -197,27 +201,27 @@ def plot_spacings(max_distance, spacing_counts):
 def yield_pairs(occurrences, seq_infos, options):
     """Yield pairs of occurrences under a given maximum distance.
     """
-    for i1, occ1 in enumerate(occurrences): # first occurrence
+    for i1, occ1 in enumerate(occurrences):  # first occurrence
         footprint1 = footprint(occ1)
         seq_length = seq_infos[occ1.seq].length
-        
+
         for i2 in xrange(i1, len(occurrences)):
             occ2 = occurrences[i2]
-    
-            if occ2.seq != occ1.seq: # only interested in occurrences on same sequence
+
+            # only interested in occurrences on same sequence
+            if occ2.seq != occ1.seq:
                 break
-            
+
             footprint2 = footprint(occ2)
             if not footprint1.disjoint(footprint2):
-                continue # not interested in overlapping occurrences
-        
+                continue  # not interested in overlapping occurrences
+
             distance = footprint2.distance(footprint1)
             if distance > options.max_distance:
-                break # only interested in pairs up to some distance apart
+                break  # only interested in pairs up to some distance apart
             assert distance in pyicl.IntInterval(0, options.max_distance + 1)
-            
-            yield seq_length, occ1, footprint1, occ2, footprint2, distance
 
+            yield seq_length, occ1, footprint1, occ2, footprint2, distance
 
 
 def make_primary_counting_handler(spacings):
@@ -228,9 +232,8 @@ def make_primary_counting_handler(spacings):
         """
         idx = spacing_idx(max_distance, distance, upstream, same_strand)
         spacings[(primary.motif, secondary.motif)][idx] += 1
-        
-    return handle_primary
 
+    return handle_primary
 
 
 def make_pair_handler_from_primary_handler(primary_handler, ignore_close_to_end, options):
@@ -241,62 +244,67 @@ def make_pair_handler_from_primary_handler(primary_handler, ignore_close_to_end,
     as they are too close to the beginning or end of a sequence.
     """
     def handle_pair(seq_length, occ1, footprint1, occ2, footprint2, distance):
-        same_strand = bool(occ1.strand == occ2.strand) # are they on the same strand?
-        
+        # are they on the same strand?
+        same_strand = bool(occ1.strand == occ2.strand)
+
         # handle occ1 as the primary occurrence
         # check whether occ1 is too close to start or end to be a primary motif
         if not ignore_close_to_end or can_be_primary(options.max_distance, footprint1, footprint2.size, seq_length):
-            primary_handler(options.max_distance, occ1, occ2, distance, occ1.strand == '-', same_strand)
-            
+            primary_handler(
+                options.max_distance, occ1, occ2, distance, occ1.strand == '-', same_strand)
+
         # handle occ2 as the primary occurrence
         # check whether occ2 is too close to start or end to be a primary motif
         if not ignore_close_to_end or can_be_primary(options.max_distance, footprint2, footprint1.size, seq_length):
-            primary_handler(options.max_distance, occ2, occ1, distance, occ2.strand == '+', same_strand)
-    
+            primary_handler(
+                options.max_distance, occ2, occ1, distance, occ2.strand == '+', same_strand)
+
     return handle_pair
-        
+
 
 def handle_pairs(occurrences, seq_infos, handler, options):
     """Handle pairs of occurrences under a given maximum distance.
     """
-    for i1, occ1 in enumerate(occurrences): # first occurrence
+    for i1, occ1 in enumerate(occurrences):  # first occurrence
         footprint1 = footprint(occ1)
         seq_length = seq_infos[occ1.seq].length
-        
+
         for i2 in xrange(i1, len(occurrences)):
             occ2 = occurrences[i2]
-    
-            if occ2.seq != occ1.seq: # only interested in occurrences on same sequence
+
+            # only interested in occurrences on same sequence
+            if occ2.seq != occ1.seq:
                 break
-            
+
             # don't use yet - check which scripts call this.
             # they all need property in options
             # ignore homodimers if requested
 #            if options.ignore_homodimers and occ1.motif == occ2.motif:
 #                continue
-            
+
             footprint2 = footprint(occ2)
             if not footprint1.disjoint(footprint2):
-                continue # not interested in overlapping occurrences
-        
+                continue  # not interested in overlapping occurrences
+
             distance = footprint2.distance(footprint1)
             if distance > options.max_distance:
-                break # only interested in pairs up to some distance apart
+                break  # only interested in pairs up to some distance apart
             assert distance in pyicl.IntInterval(0, options.max_distance + 1)
 
             handler(seq_length, occ1, footprint1, occ2, footprint2, distance)
 
 
-
 def count_all_pairs(occurrences, seq_infos, ignore_close_to_end, options):
     """Count all pairs of motifs in the occurrences.
     """
-    spacings = defaultdict(lambda: numpy.zeros(4 * (options.max_distance+1), dtype=numpy.uint))
+    spacings = defaultdict(lambda: numpy.zeros(
+        4 * (options.max_distance + 1), dtype=numpy.uint))
     primary_handler = make_primary_counting_handler(spacings)
-    pair_handler = make_pair_handler_from_primary_handler(primary_handler, ignore_close_to_end, options)
+    pair_handler = make_pair_handler_from_primary_handler(
+        primary_handler, ignore_close_to_end, options)
     handle_pairs(occurrences, seq_infos, pair_handler, options)
     return spacings
-    
+
 
 def parse_spacings(f):
     """Parse a list of spacings. Returns a map from motif pairs to sets of spacings
@@ -304,8 +312,9 @@ def parse_spacings(f):
     result = defaultdict(list)
     for line in f:
         if line.startswith('#'):
-            continue # ignore comments
-        primary, secondary, same_strand, upstream, distance = line.strip().split()
+            continue  # ignore comments
+        primary, secondary, same_strand, upstream, distance = line.strip(
+        ).split()
         distance = int(distance)
         if 'S' != same_strand and 'C' != same_strand:
             raise ValueError('Expecting "S" or "C" for same strand field')
@@ -314,12 +323,15 @@ def parse_spacings(f):
             raise ValueError('Expecting "U" or "D" for upstream field')
         upstream = 'U' == upstream
         result[primary, secondary].append(
-            Spacing(primary=primary, secondary=secondary, same_strand=same_strand, upstream=upstream, distance=distance)
+            Spacing(primary=primary, secondary=secondary,
+                    same_strand=same_strand, upstream=upstream, distance=distance)
         )
     return result
 
 
-spacing_header = '%30s %-25s %s %2s %4s' % ('Primary', 'Secondary', 'Strand', 'Up', 'Dist')
+spacing_header = '%30s %-25s %s %2s %4s' % (
+    'Primary', 'Secondary', 'Strand', 'Up', 'Dist')
+
 
 def spacing_str(spacing):
     return '%30s %-30s %s %2s %4d' % (
@@ -332,7 +344,9 @@ def check_spacings(motif_pairs, motifs):
         if primary not in motifs:
             raise ValueError('Motif "%s" is not in the occurrences' % primary)
         if secondary not in motifs:
-            raise ValueError('Motif "%s" is not in the occurrences' % secondary)
+            raise ValueError(
+                'Motif "%s" is not in the occurrences' % secondary)
+
 
 def log_spacings(spacings):
     """Log the spacings
@@ -353,5 +367,3 @@ def load_spacings(spacings_filename, motifs, options):
     check_spacings(spacings.keys(), motifs)
     log_spacings(spacings.values())
     return spacings
-
-
