@@ -28,7 +28,7 @@ def update_path_for_stempy():
     prepend_to_path(os.path.normpath(os.path.join(d, '..', '..', '..', '..', 'PyICL', 'Python'))) # PyICL
     prepend_to_path(os.path.normpath(os.path.join(d, '..', '..', '..', '..', 'Python', 'Cookbook', 'python'))) # cookbook
     prepend_to_path(os.path.normpath(os.path.join(d, '..', '..'))) # stempy
-    print sys.path
+    #print sys.path
 
 def fasta_dir():
     return os.path.normpath(os.path.join(os.path.dirname(__file__), 'fasta'))
