@@ -17,7 +17,7 @@ import os, sys, stempy
 # Run STEME
 #
 options = stempy.get_default_options()
-options.output_dir = os.path.join('output', 'test-run-steme')
+options.output_dir = os.path.join('output', 'test-run-steme-with-bg')
 options.min_w = options.max_w = 8
 options.write_em_stats = True
 options.meme_like_output = 'meme.out'
